export type Comparator<T> = (a: T, b: T) => number;

export function reverse<T>(cmp: Comparator<T>): Comparator<T> {
  return (a, b) => cmp(a, b) * -1;
}

export function sorted<T>(arr: Readonly<T[]>, cmp: Comparator<T>): T[] {
  return arr.slice().sort(cmp);
}

export function compose<T>(...cmps: Comparator<T>[]): Comparator<T> {
  return (a: T, b: T): number => {
    let val: number;
    for (let i = 0; i < cmps.length; i++) {
      const cmp = cmps[i];
      val = cmp(a, b);
      if (val !== 0) {
        return val;
      }
    }
    return 0;
  };
}
